@@ -13,9 +13,10 @@ from trytond.modules.currency.fields import Monetary
 from trytond.modules.product import round_price
 from trytond.modules.sale.exceptions import SaleConfirmError
 from trytond.pool import Pool, PoolMeta
+from trytond.pyson import Eval
 from trytond.transaction import Transaction
 
-from .common import IdentifierMixin
+from .common import IdentifierMixin, setattr_changed
 from .exceptions import ShopifyError
 
 
@@ -57,29 +58,44 @@ class Sale(IdentifierMixin, metaclass=PoolMeta):
             party.save()
             party.set_shopify_identifier(shop, order.customer.id)
         else:
-            party = Party()
-            party.save()
+            party = shop.guest_party
 
         if not sale:
             sale = shop.get_sale(party=party)
             sale.shopify_identifier = order.id
+        setattr_changed(sale, 'party', party)
+
         assert sale.shopify_identifier == order.id
         if order.location_id:
             for shop_warehouse in shop.shopify_warehouses:
                 if shop_warehouse.shopify_id == str(order.location_id):
                     sale.warehouse = shop_warehouse.warehouse
                     break
-        if sale.currency.code != order.currency:
+        if sale.currency.code != order.presentment_currency:
             sale.currency, = Currency.search([
-                    ('code', '=', order.currency),
+                    ('code', '=', order.presentment_currency),
                     ], limit=1)
 
-        if getattr(order, 'shipping_address', None):
-            sale.shipment_address = party.get_address_from_shopify(
-                order.shipping_address)
-        if getattr(order, 'billing_address', None):
-            sale.invoice_address = party.get_address_from_shopify(
-                order.billing_address)
+        if sale.party != shop.guest_party:
+            if getattr(order, 'shipping_address', None):
+                shipment_address = party.get_address_from_shopify(
+                    order.shipping_address)
+            else:
+                shipment_address = None
+            if getattr(order, 'billing_address', None):
+                invoice_address = party.get_address_from_shopify(
+                    order.billing_address)
+            else:
+                invoice_address = None
+        else:
+            shipment_address = sale.party.address_get(type='delivery')
+            invoice_address = sale.party.address_get(type='invoice')
+
+        if shipment_address:
+            setattr_changed(sale, 'shipment_address', shipment_address)
+        if invoice_address or shipment_address:
+            setattr_changed(
+                sale, 'invoice_address', invoice_address or shipment_address)
 
         if not party.addresses:
             address = Address(party=party)
@@ -89,20 +105,22 @@ class Sale(IdentifierMixin, metaclass=PoolMeta):
             if not sale.invoice_address:
                 sale.invoice_address = address
 
-        sale.reference = order.name
-        sale.comment = order.note
-        sale.sale_date = dateutil.parser.isoparse(
-            order.processed_at or order.created_at).date()
+        setattr_changed(sale, 'reference', order.name)
+        setattr_changed(sale, 'comment', order.note)
+        setattr_changed(sale, 'sale_date', dateutil.parser.isoparse(
+                order.processed_at or order.created_at).date())
 
         if order.phone:
             for contact_mechanism in party.contact_mechanisms:
                 if (contact_mechanism.type in {'phone', 'mobile'}
-                        and contact_mechanism.value == order.phone):
+                        and (contact_mechanism.value_compact
+                            == contact_mechanism.format_value_compact(
+                                order.phone, contact_mechanism.type))):
                     break
             else:
                 contact_mechanism = ContactMechanism(
                     party=party, type='phone', value=order.phone)
-            sale.contact = contact_mechanism
+            setattr_changed(sale, 'contact', contact_mechanism)
 
         refund_line_items = defaultdict(list)
         for refund in order.refunds:
@@ -121,8 +139,7 @@ class Sale(IdentifierMixin, metaclass=PoolMeta):
             line = id2line.pop(line_item.id, None)
             quantity = line_item.quantity
             for refund_line_item in refund_line_items[line_item.id]:
-                if refund_line_item.restock_type == 'cancel':
-                    quantity -= refund_line_item.quantity
+                quantity -= refund_line_item.quantity
             lines.append(Line.get_from_shopify(
                     sale, line_item, quantity, line=line))
         for shipping_line, line in zip_longest(
@@ -218,10 +235,10 @@ class Sale(IdentifierMixin, metaclass=PoolMeta):
 
             # TODO: manage drop shipment
 
-            if self.shipment_state == 'sent':
+            if self.shipment_state == 'sent' or self.state == 'done':
                 # TODO: manage shopping refund
                 refund = self.get_shopify_refund(shipping={
-                        'full_refund': False,
+                        'full_refund': self.shipment_state == 'none',
                         })
                 if refund:
                     if not refund.save():
@@ -267,7 +284,10 @@ class Sale(IdentifierMixin, metaclass=PoolMeta):
                 location_id = shop_warehouse.shopify_id
         for line in self.lines:
             if (line.type != 'line'
-                    or not line.shopify_identifier):
+                    or not line.shopify_identifier
+                    or not (line.product
+                        and line.product.type in
+                        line.get_move_product_types())):
                 continue
             fulfillable_quantity = fulfillable_quantities.get(
                 line.shopify_identifier, 0)
@@ -301,29 +321,44 @@ class Sale_ShipmentCost(metaclass=PoolMeta):
 
         sale = super().get_from_shopify(shop, order, sale=sale)
 
-        sale.shipment_cost_method = 'order'
+        shipment_cost_method = None
         if order.shipping_lines:
             available_carriers = sale.on_change_with_available_carriers()
+            carrier = None
             if available_carriers:
-                sale.carrier = available_carriers[0]
+                carrier = available_carriers[0]
+            setattr_changed(sale, 'carrier', carrier)
             if sale.carrier:
+                shipment_cost_method = 'order'
                 for line in sale.lines:
                     if getattr(line, 'shipment_cost', None) is not None:
                         unit_price = line.unit_price
                         base_price = getattr(line, 'base_price', None)
-                        line.product = sale.carrier.carrier_product
-                        line.on_change_product()
-                        line.unit_price = round_price(Tax.reverse_compute(
+                        if setattr_changed(
+                                line, 'product', sale.carrier.carrier_product):
+                            line.on_change_product()
+                        unit_price = round_price(Tax.reverse_compute(
                                 unit_price, line.taxes, sale.sale_date))
+                        setattr_changed(line, 'unit_price', unit_price)
                         if base_price is not None:
-                            line.base_price = round_price(Tax.reverse_compute(
+                            base_price = round_price(Tax.reverse_compute(
                                     base_price, line.taxes, sale.sale_date))
+                            setattr_changed(line, 'base_price', base_price)
+        setattr_changed(sale, 'shipment_cost_method', shipment_cost_method)
         return sale
 
 
 class Line(IdentifierMixin, metaclass=PoolMeta):
     __name__ = 'sale.line'
 
+    @classmethod
+    def __setup__(cls):
+        super().__setup__()
+        cls.product.states['readonly'] = (
+            cls.product.states['readonly']
+            & ~((Eval('sale_state') == 'confirmed')
+                & Eval('shopify_identifier')))
+
     @classmethod
     def get_from_shopify(cls, sale, line_item, quantity, line=None):
         pool = Pool()
@@ -334,50 +369,53 @@ class Line(IdentifierMixin, metaclass=PoolMeta):
             line = cls(type='line')
             line.sale = sale
             line.shopify_identifier = line_item.id
+            line.product = None
         assert line.shopify_identifier == line_item.id
         if getattr(line_item, 'variant_id', None):
-            line.product = Product.search_shopify_identifier(
-                sale.web_shop, line_item.variant_id)
-        else:
-            line.product = None
+            if product := Product.search_shopify_identifier(
+                    sale.web_shop, line_item.variant_id):
+                setattr_changed(line, 'product', product)
         if line.product:
             line._set_shopify_quantity(line.product, quantity)
-            line.on_change_product()
+            if line._changed_values:
+                line.on_change_product()
         else:
-            line.quantity = quantity
-            line.description = line_item.title
-            line.taxes = []
+            setattr_changed(line, 'quantity', quantity)
+            setattr_changed(line, 'description', line_item.title)
+            setattr_changed(line, 'taxes', ())
         total_discount = sum(
             Decimal(d.amount) for d in line_item.discount_allocations)
         unit_price = ((
-                (Decimal(line_item.price) * line_item.quantity)
+                (Decimal(line_item.price) * (line_item.quantity or 1))
                 - Decimal(total_discount))
-            / line_item.quantity)
+            / (line_item.quantity or 1))
         unit_price = round_price(Tax.reverse_compute(
                 unit_price, line.taxes, sale.sale_date))
         if line.product:
             line._set_shopify_unit_price(line.product, unit_price)
         else:
-            line.unit_price = unit_price
+            setattr_changed(line, 'unit_price', unit_price)
         return line
 
     def _set_shopify_quantity(self, product, quantity):
         if product.shopify_uom.category == product.sale_uom.category:
-            self.unit = self.product.shopify_uom
-            self.quantity = quantity
+            setattr_changed(self, 'unit', self.product.shopify_uom)
+            setattr_changed(self, 'quantity', quantity)
 
     def _set_shopify_unit_price(self, product, unit_price):
         if product.shopify_uom.category == product.sale_uom.category:
-            self.unit_price = unit_price
+            setattr_changed(self, 'unit_price', unit_price)
 
     @classmethod
     def get_from_shopify_shipping(cls, sale, shipping_line, line=None):
         if not line:
             line = cls(type='line')
             line.sale = sale
-        line.quantity = 1
-        line.unit_price = round_price(Decimal(shipping_line.discounted_price))
-        line.description = shipping_line.title
+        setattr_changed(line, 'quantity', 1)
+        setattr_changed(
+            line, 'unit_price',
+            round_price(Decimal(shipping_line.discounted_price)))
+        setattr_changed(line, 'description', shipping_line.title)
         return line
 
     def _get_invoice_line_quantity(self):
@@ -398,15 +436,16 @@ class Line_Discount(metaclass=PoolMeta):
         pool = Pool()
         Tax = pool.get('account.tax')
         line = super().get_from_shopify(sale, line_item, quantity, line=line)
-        line.base_price = round_price(Tax.reverse_compute(
-                Decimal(line_item.price), line.taxes, sale.sale_date))
+        setattr_changed(line, 'base_price', round_price(
+                Tax.reverse_compute(
+                    Decimal(line_item.price), line.taxes, sale.sale_date)))
         return line
 
     @classmethod
     def get_from_shopify_shipping(cls, sale, shipping_line, line=None):
         line = super().get_from_shopify_shipping(
             sale, shipping_line, line=line)
-        line.base_price = Decimal(shipping_line.price)
+        setattr_changed(line, 'base_price', Decimal(shipping_line.price))
         return line
 
 
@@ -418,19 +457,21 @@ class Line_SaleSecondaryUnit(metaclass=PoolMeta):
         if (product.sale_secondary_uom
                 and product.shopify_uom.category
                 == product.sale_secondary_uom.category):
-            self.unit = product.sale_uom
-            self.secondary_unit = product.shopify_uom
-            self.on_change_product()
-            self.secondary_quantity = quantity
-            self.on_change_secondary_quantity()
+            changed = setattr_changed(self, 'unit', product.sale_uom)
+            changed |= setattr_changed(
+                self, 'secondary_unit', product.shopify_uom)
+            if changed:
+                self.on_change_product()
+            if setattr_changed(self, 'secondary_quantity', quantity):
+                self.on_change_secondary_quantity()
 
     def _set_shopify_unit_price(self, product, unit_price):
         super()._set_shopify_unit_price(product, unit_price)
         if (product.sale_secondary_uom
                 and product.shopify_uom.category
                 == product.sale_secondary_uom.category):
-            self.secondary_unit_price = unit_price
-            self.on_change_secondary_unit_price()
+            if setattr_changed(self, 'secondary_unit_price', unit_price):
+                self.on_change_secondary_unit_price()
 
 
 class Line_ShipmentCost(metaclass=PoolMeta):
@@ -440,7 +481,7 @@ class Line_ShipmentCost(metaclass=PoolMeta):
     def get_from_shopify_shipping(cls, sale, shipping_line, line=None):
         line = super().get_from_shopify_shipping(
             sale, shipping_line, line=line)
-        line.shipment_cost = Decimal(shipping_line.price)
+        setattr_changed(line, 'shipment_cost', Decimal(shipping_line.price))
         return line
 
 # TODO: refund as return sale
