@@ -2,6 +2,7 @@
 # this repository contains the full copyright notices and license terms.
 import datetime as dt
 import urllib.parse
+from collections import defaultdict
 from decimal import Decimal
 from operator import attrgetter
 
@@ -502,11 +503,16 @@ class Shop(metaclass=PoolMeta):
                 Sale.save(sales)
                 for sale, order in zip(sales, orders):
                     sale.shopify_tax_adjustment = (
-                        Decimal(order.total_price) - sale.total_amount)
+                        Decimal(order.current_total_price) - sale.total_amount)
                 Sale.save(sales)
-                Sale.quote(sales)
+                to_quote = [
+                    s for s in sales if s.party != s.web_shop.guest_party]
+                if to_quote:
+                    Sale.quote(to_quote)
                 for sale, order in zip(sales, orders):
-                    Payment.get_from_shopify(sale, order)
+                    if sale.state != 'draft':
+                        Payment.get_from_shopify(sale, order)
+                Sale.payment_confirm(sales)
 
     @classmethod
     def shopify_update_order(cls, shops=None):
@@ -562,6 +568,7 @@ class Shop(metaclass=PoolMeta):
         Sale = pool.get('sale.sale')
         assert len(sales) == len(orders)
         to_update = {}
+        states_to_restore = defaultdict(list)
         for sale, order in zip(sales, orders):
             assert sale.shopify_identifier == order.id
             shop = sale.web_shop
@@ -571,15 +578,40 @@ class Shop(metaclass=PoolMeta):
                     sale.untaxed_amount_cache = None
                     sale.tax_amount_cache = None
                     sale.total_amount_cache = None
+                    sale.shopify_tax_adjustment = None
                     to_update[sale] = order
-                Payment.get_from_shopify(sale, order)
+                    states_to_restore[sale.state].append(sale)
+        Sale.write(list(to_update.keys()), {'state': 'draft'})
         Sale.save(to_update.keys())
+        for state, state_sales in states_to_restore.items():
+            Sale.write(list(state_sales), {'state': state})
         for sale, order in to_update.items():
             sale.shopify_tax_adjustment = (
                 Decimal(order.current_total_price) - sale.total_amount)
+        Sale.save(to_update.keys())
         Sale.store_cache(to_update.keys())
         Amendment._clear_sale(to_update.keys())
-        Sale.__queue__.process(to_update.keys())
+        to_process, to_quote = [], []
+        for sale in to_update:
+            if sale.party == sale.web_shop.guest_party:
+                continue
+            if sale.payment_amount_authorized >= sale.amount_to_pay:
+                to_process.append(sale)
+            else:
+                to_quote.append(sale)
+        if to_process:
+            Sale.__queue__.process(to_process)
+        if to_quote:
+            # Use write because there is no transition
+            Sale.write(to_quote, {'state': 'quotation'})
+            cls.log(to_quote, 'transition', 'state:quotation')
+
+        for sale, order in zip(sales, orders):
+            if sale.state != 'draft':
+                shop = sale.web_shop
+                with shop.shopify_session():
+                    Payment.get_from_shopify(sale, order)
+        Sale.payment_confirm(sales)
 
 
 class ShopShopifyIdentifier(IdentifierMixin, ModelSQL, ModelView):
